<?php
##############################################################
# FILENAME  : module_loader.php
# COPYRIGHT : (c) 2006, ShuttleBB Development Team <developers@shuttlebb.com>
# GPL v2.0 or later. See docs/COPYING.
##############################################################
# This program is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA
##############################################################
/**
 * Module initialization class
 * This class finds and loads modules securely based on the url and module name.
 */
class module_loader
{
	/**
	 * Contains the entire object of the loaded module
	 *
	 * @var object
	 */
	var $module;

	/**
	 * Contains the module directory
	 *
	 * @var string
	 */
	var $mod_dir;

	/**
	 * Contains a list of active modules
	 *
	 * @var array
	 */
	var $mod_list;

	/**
	 * Contains the name of the module that is currently active
	 *
	 * @var string
	 */
	var $mod_name;

	/**
	 * Constructor for the module loader class
	 *
	 * @param string $mod_dir
	 * @return object
	 */
	function module_loader ( $mod_dir = './' )
	{
		$this->mod_dir	= $mod_dir;
		$this->get_modules ( );
	}

	/**
	 * Finds, Includes and Executes the active module
	 *
	 * @param string $mod_name
	 */
	function load_module ( $mod_name )
	{
		// Check for pre-generated module list
		if ( !is_array ( $this->mod_list ) )
		{
			// If not, walk modules dir.
			$this->mod_list	= $this->get_modules ( );
		}

		// Include the module and eval for init.
		$this->mod_name	= $mod_name;
		include ( $this->mod_dir . 'mod_' . $mod_name . '.php' );
		eval ( '$this->module = new mod_' . $mod_name . '();' );

		return;
	}

	/**
	 * Walks the modules directory building a list of active modules
	 *
	 * @param string $specific
	 * @return array
	 */
	function get_modules ( $specific = '' )
	{
		// Init variable.
		$this->mod_list	= array ( );

		// Walk through modules directory.
		$dir	= @opendir ( $this->mod_dir );
		while ( $file = basename ( @readdir ( $dir ) ) )
		{
			if ( preg_match ( '#mod_' . $specific . '(.*?)\.php#is', $file, $match ) )
			{
				$this->mod_list[ ]	= $match[ 1 ];
			}
		}

		// Sort by filename in alphabetical order.
		sort ( $this->mod_list );

		// Return list.
		return $this->mod_list;
	}

	/**
	 * Checks for dependancies.
	 *
	 * @param boolean $get
	 * @param string $for
	 * @return mixed
	 */
	function deps ( $get = false, $for = '' )
	{
		global	$db;
		global	$http_vars;
		global	$lang;

		if ( !empty ( $for ) )
		{
			$use	= $for;
		}
		else
		{
			$use	= ( !empty ( $http_vars[ 'sub_module' ] ) ) ? $http_vars[ 'sub_module' ] : $http_vars[ 'module' ];
		}

		// Run the query to get the module details.
		$sql	= 'SELECT * FROM ' . MODULES_TABLE . ' WHERE module_name = "' . $use . '"';
		$result	= $db->sql_query ( $sql );
		$row	= $db->sql_fetchrow ( $result );

		// Any dependancies ?
		if ( !empty ( $row[ 'module_deps' ] ) )
		{
			if ( $get == false )
			{
				// Yes. Go though each dependancy.
				foreach ( explode ( '|', $row[ 'module_deps' ] ) AS $dep )
				{
					// Check if the dependancies are disabled.
					$sql_2		= 'SELECT * FROM ' . MODULES_TABLE . ' WHERE module_name = "' . $dep . '" AND module_status = 1';
					if ( !$db->sql_query ( $sql_2 ) )
					{
						// The dependancy is disabled, kill the module.
						message_die ( GENERAL_ERROR, sprintf ( $lang[ 'dothdidsa' ], $use, $dep ) );
						return false;
					}
					else
					{
						// It is enabled, continue.
						continue;
					}
				}
			}
			else
			{
				// Return it's dependancies.
				return explode ( '|', $row[ 'module_deps' ] );
			}
		}

		return true;
	}
}

?>
