<?php
##############################################################
# FILENAME  : mod_ladder.php
# COPYRIGHT : (c) 2006, Tyler N. King <aibotca@yahoo.ca>
# GPL v2.0 or later. See docs/COPYING.
##############################################################
# This program is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA
##############################################################
/**
 * ACP Ladder Class.
 * Shows the platforms, ladders/sub-ladders and standings.
 */
class mod_ladder
{
	/**
	 * Initilizes the class and displays the module.
	 */
	function mod_ladder ( )
	{
		// Include the globals needed to run this module.
		global	$db;
		global	$http_vars;
		global	$userdata;
		global	$lang;
		global	$template;
		global	$ladder;
		global	$board_config;

		// Are we loading a sub-module ?
		if ( !empty ( $http_vars[ 'sub_module' ] ) )
		{
			// Get the modules from the database.
			$sql	= 'SELECT * FROM ' . MODULES_TABLE . ' WHERE module_name = "' . $http_vars[ 'sub_module' ] . '"';
			$result	= $db->sql_query ( $sql );
			$row	= $db->sql_fetchrow ( $result );

			// Check if we can load the module.
			if ( $row[ 'module_status' ] != 0 )
			{
				// Yes, load it.
				$mod	= new module_loader ( 'rivals/modules/' );
				$mod->deps ( );
				$mod->load_module ( $http_vars[ 'sub_module' ] );
			}
			else
			{
				$mod	= new module_loader ( 'rivals/modules/' );
				$mod->deps ( );
				$mod->load_module ( $http_vars[ 'module' ] );
			}
		}
		else
		{
			// Pagination variable setup.
			$http_vars[ 'start' ]	= ( !empty ( $http_vars[ 'start' ] ) ) ? $http_vars[ 'start' ] : 0;

			// Are we going to show the rules ?
			if ( !empty ( $http_vars[ 'rules' ] ) )
			{
				// Yes, load the rules from the ladder class.
				$rules	= $ladder->data[ 'ladder_rules' ];
				if ( $board_config[ 'allow_bbcode' ] )
				{
					// Parse the text: bbcode, emotions, new lines to HTML breaks.
					$rules_uid	= make_bbcode_uid ( );
					$rules		= bbencode_first_pass ( $rules, $rules_uid );
					$rules		= bbencode_second_pass ( $rules, $rules_uid );
					$rules		= smilies_pass ( $rules );
					$rules		= nl2br ( $rules );
				}

				// Show the parsed rules.
				message_die ( GENERAL_MESSAGE, $rules );
			}

			// Is the platform set ?
			if ( !empty ( $http_vars[ 'platform' ] ) )
			{
				// Is the ladder_id set ?
				if ( !empty ( $http_vars[ 'ladder_id' ] ) )
				{
					// Yes, ladder_id is set, load the clans joined to it.
					$sql		= 'SELECT count(*) AS total FROM ' . CLANS_TABLE . ' WHERE ladder_joined = ' . $http_vars[ 'ladder_id' ];
					$result	= $db->sql_query ( $sql );

					if ( $total = $db->sql_fetchrow ( $result ) )
					{
						// Setup the pagination.
						$total_pag_items	= $total[ 'total' ];
						$pagination 		= generate_pagination ( 'rivals.php?module=ladders&platform=' . $http_vars[ 'platform' ] . '&ladder_id=' . $http_vars[ 'ladder_id' ], $total_pag_items, $board_config[ 'topics_per_page' ], $http_vars[ 'start' ] );
					}

					// Assign the variables to the clan block.
					$template->assign_block_vars ( 'block_clan', array (
						'PAGINATION' => $pagination,
						'PAGE_NUMBER' => sprintf ( $lang[ 'Page_of' ], ( floor (  $start / $board_config[ 'topics_per_page' ] ) + 1 ), ceil (  $total_pag_items / $board_config[ 'topics_per_page' ] ) ),
						'L_SORTBY' => $lang[ 'rivals_sort' ],
						'S_CLANRATIO' => 'clan_ratio',
						'S_CLANSCORE' => 'clan_score',
						'L_CLANID' => $lang[ 'rivals_id' ],
						'L_CLANNAME' => $lang[ 'rivals_cnop' ],
						'L_CLANRATIO' => $lang[ 'rivals_rat' ],
						'L_CLANSCORE' => $lang[ 'rivals_sbs' ],
						'L_CLANWINS' => $lang[ 'rivals_opcw' ],
						'L_CLANLOSS' => $lang[ 'rivals_opcl' ],
						'L_CLANTIES' => $lang[ 'ocmgwhast' ],
						'L_CLANSTREAK' => $lang[ 'rivals_sop' ],
						'L_POS' => $lang[ 'rivals_pos' ] )
					);

					// Order the clans by their score.
					$sql	= 'SELECT * FROM ' . CLANS_TABLE . ' WHERE ladder_joined = "' . $http_vars[ 'ladder_id' ] . '" ORDER BY clan_score DESC LIMIT ' . $http_vars[ 'start' ] . ', ' . $board_config[ 'topics_per_page' ];
					$result	= $db->sql_query ( $sql );

					// Order clans by rating ?
					$ladder_rating	= $ladder->data ( 'ladder_rating', $http_vars[ 'ladder_id' ] );
					if ( $ladder_rating == 1 )
					{
						// This crap gets complex now, sorting an array by a key and values.
						$data	= array ( );
						for ( $i = 1; $i <= $db->sql_numrows ( $result ); $i++ )
						{
							$row	= $db->sql_fetchrow ( $result );

							// Rating: ( wins - losses ) + 100
							$data[ ]	= array (
								'clan_id' => $row[ 'clan_id' ],
								'clan_name' => $row[ 'clan_name' ],
								'clan_win' => $row[ 'clan_win' ],
								'clan_loss' => $row[ 'clan_loss' ],
								'clan_ties' => $row[ 'clan_ties' ],
								'clan_score' => $row[ 'clan_score' ],
								'clan_streak' => $row[ 'clan_streak' ],
								'clan_rating' => ( $row[ 'clan_win' ] - $row[ 'clan_loss' ] ) + 100
							);
						}

						// Sort it by the field.
						$new_data	= sort_by ( $data, 'clan_rating' );
						for ( $i = 0; $i < count ( $new_data ); $i++ )
						{
							// Ugh, apply the proper ladder position to the clan
							if ( $http_vars[ 'start' ] > 0 )
							{
								$pos	= $http_vars[ 'start' ] + ( $i + 1 );
							}
							else
							{
								$pos	= $i + 1;
							}

							// Assign each clan to the template.
							$template->assign_block_vars ( 'block_clan.block_clans', array (
								'S_CLANNAME' => $new_data[ $i ][ 'clan_name' ],
								'S_CLANID' => $new_data[ $i ][ 'clan_id' ],
								'S_CLANWINS' => $new_data[ $i ][ 'clan_win' ],
								'S_ROWCOLOR' => ( $i % 2 ) ? 'row1' : 'row2',
								'U_ACTION' => append_sid ( 'rivals.php?module=ladder&sub_module=ladderviewclan&clan_id=' . $new_data[ $i ][ 'clan_id' ] ),
								'S_CLANLOSS' => $new_data[ $i ][ 'clan_loss' ],
								'S_CLANTIES' => $new_data[ $i ][ 'clan_ties' ],
								'S_CLANSCORE' => $new_data[ $i ][ 'clan_score' ],
								'S_CLANSTREAK' => $new_data[ $i ][ 'clan_streak' ],
								'I_UPODOWN' => $image,
								'S_POS' => $pos )
							);
						}
					}
					else
					{
						// No, order by score.
						for ( $i = 0; $i < $db->sql_numrows ( $result ); $i++ )
						{
							$row	= $db->sql_fetchrow ( $result );

							// Assign the clans to the template.
							$template->assign_block_vars ( 'block_clan.block_clans', array (
								'S_CLANNAME' => $row[ 'clan_name' ],
								'S_CLANID' => $row[ 'clan_id' ],
								'S_CLANWINS' => $row[ 'clan_win' ],
								'S_ROWCOLOR' => ( $i % 2 ) ? 'row1' : 'row2',
								'U_ACTION' => append_sid ( 'rivals.php?module=ladder&sub_module=ladderviewclan&clan_id=' . $row[ 'clan_id' ] ),
								'S_CLANLOSS' => $row[ 'clan_loss' ],
								'S_CLANTIES' => $row[ 'clan_ties' ],
								'S_CLANSCORE' => $row[ 'clan_score' ],
								'S_CLANSTREAK' => $row[ 'clan_streak' ],
								'I_UPODOWN' => $image,
								'S_POS' => $pos )
							);
						}
					}
				}
				else
				{
					// No, the ladder_id is not set.
					$template->assign_block_vars ( 'block_ladder', array (
						'L_LADDERNAME' => $lang[ 'rivals_lnop' ],
						'L_NUMCLANS' => $lang[ 'rivals_mo6' ] )
					);

					// Get the parent ladders and order them.
					$sql	= 'SELECT * FROM ' . LADDERS_TABLE . ' WHERE (ladder_parent = "" OR ladder_parent = "0") AND platform = "' . $http_vars[ 'platform' ] . '" ORDER BY ladder_order ASC';
					$result	= $db->sql_query ( $sql );

					for ( $i = 0; $i < $db->sql_numrows ( $result ); $i++ )
					{
						$row	= $db->sql_fetchrow ( $result );

						// Assign the ladders to the template.
						$template->assign_block_vars ( 'block_ladder.block_ladders', array (
							'S_LADDERNAME' => $row[ 'ladder_name' ],
							'U_LADDERRULES' => append_sid ( 'rivals.php?module=ladder&rules=1&ladder_id=' . $row[ 'ladder_id' ] ) )
						);

						// Get the sub-ladders and order them.
						$sql_2		= 'SELECT * FROM ' . LADDERS_TABLE . ' WHERE ladder_parent = "' . $row[ 'ladder_id' ] . '" ORDER BY subladder_order ASC';
						$result_2	= $db->sql_query ( $sql_2 );

						for ( $x = 0; $x < $db->sql_numrows ( $result_2 ); $x++ )
						{
							$row_2	= $db->sql_fetchrow ( $result_2 );

							// Count the number of clans in the sub-ladder.
							$sql_3		= 'SELECT count(*) AS num_clans FROM ' . CLANS_TABLE . ' WHERE ladder_joined = ' . $row[ 'ladder_id' ];
							$result_3	= $db->sql_query ( $sql_3 );
							$row_3		= $db->sql_fetchrow ( $result_3 );

							// Is bbcode allowed ?
							if ( $board_config[ 'allow_bbcode' ] )
							{
								// Yes. Parse the text: bbcode, emotions, new lines to HTML breaks.
								$desc_uid	= make_bbcode_uid ( );
								$desc		= bbencode_first_pass ( $row_2[ 'ladder_desc' ], $desc_uid );
								$desc		= bbencode_second_pass ( $desc, $desc_uid );
								$desc		= smilies_pass ( $desc );
								$desc		= nl2br ( $desc );
							}

							// Assign the sub-ladders to the template.
							$template->assign_block_vars ( 'block_ladder.block_ladders.block_subladders', array (
								'S_LADDERNAME' => $row_2[ 'ladder_name' ],
								'S_NUMCLANS' => $row_3[ 'num_clans' ],
								'U_ACTION' => append_sid ( 'rivals.php?module=ladder&platform=' . $http_vars[ 'platform' ] . '&ladder_id=' . $row_2[ 'ladder_id' ] ),
								'S_LADDERDESC' => $desc )
							);
						}
					}
				}
			}
			else
			{
				// No, the platform is not set.
				$template->assign_block_vars ( 'block_platform', array (
					'L_PLATFORMS' => $lang[ 'rivals_pop' ],
					'L_PLATFORM' => $lang[ 'rivals_yiyb' ],
					'L_NUMLADDERS' => $lang['Ladders'],
					'L_PLATFORMDESC' => $lang[ 'rivals_yiyd' ] )
				);

				// Get the platforms.
				$sql	= 'SELECT * FROM ' . PLATFORMS_TABLE;
				$result	= $db->sql_query ( $sql );

				for ( $i = 1; $i <= $db->sql_numrows ( $result ); $i++ )
				{
					$row	= $db->sql_fetchrow ( $result );

					// Count the number of ladders for each platform.
					$sql_2		= 'SELECT count(*) AS num_ladders FROM ' . LADDERS_TABLE . ' WHERE platform = ' . $row[ 'platform_id' ];
					$result_2	= $db->sql_query ( $sql_2 );
					$row_2		= $db->sql_fetchrow ( $result_2 );

					// Assign each platform to the template.
					$template->assign_block_vars ( 'block_platform.block_platforms', array (
						'S_ROWCOLOR' => ( $i % 2 ) ? 'row1' : 'row2',
						'S_COUNT' => $i,
						'S_NUMLADDERS' => $row_2[ 'num_ladders' ],
						'S_PLATFORMNAME' => $row[ 'platform_name' ],
						'U_ACTION' => append_sid ( 'rivals.php?module=ladder&platform=' . $row[ 'platform_id' ] ) )
					);
				}
			}

			$template->set_filenames ( array (
				'body' => 'rivals/mod_ladder.tpl' )
			);
			$template->pparse ( 'body' );
		}
	}
}

?>
